// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using System.Linq;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;

namespace DsPdfWeb.Demos.Basics
{
    // このサンプルは、読み込まれた PDF で見つかったすべてのフォントを一覧表示し、
    // 各フォントの情報をいくつか印刷して、
    // PDF のフォントから Font オブジェクトを作成できるかどうかを示します。
    public class ListFonts
    {
        public int CreatePDF(Stream stream)
        {
            var sourcePDF = "CompleteJavaScriptBook.pdf";

            var doc = new GcPdfDocument();
            var page = doc.NewPage();

            var rc = Common.Util.AddNote(
                "このサンプルは任意の PDF を一時的な GcPdfDocument に読み込み、その文書で" +
                "見つかったすべてのフォントをそれらのいくつかのプロパティとともに一覧表示" +
                "します。また、それらの各 PDF フォントから Font オブジェクトの作成を試み、" +
                "この操作が成功したかどうかを報告します。",
                page);

            // テキストを描画するためのテキストレイアウト。
            var tab = 24;
            var tl = page.Graphics.CreateTextLayout();
            tl.DefaultFormat.Font = Common.Util.getFont();
            tl.DefaultFormat.FontSize = 12;
            tl.MaxWidth = doc.PageSize.Width;
            tl.MaxHeight = doc.PageSize.Height;
            tl.MarginAll = rc.Left;
            tl.MarginTop = rc.Bottom + 36;
            tl.TabStops = new List<TabStop>()
            {
                new TabStop(tab)
            };
            tl.FirstLineIndent = -tab;
            tl.MarginRight = 72;

            // widow/orphan 制御のテキスト分割オプション。
            var to = new TextSplitOptions(tl)
            {
                KeepParagraphLinesTogether = true,
                MinLinesInFirstParagraph = 2,
                MinLinesInLastParagraph = 2,
                RestMarginTop = rc.Left,
            };

            // 任意の PDF を開き、一時的なドキュメントに読み込んで、すべてのフォントを取得します。
            using var fs = File.OpenRead(Path.Combine("Resources", "PDFs", sourcePDF));
                var doc1 = new GcPdfDocument();
                doc1.Load(fs);

                var fonts = doc1.GetFonts();

                tl.AppendLine($"「{sourcePDF}」で合計 {fonts.Count} 種のフォントが使用されています。");
                tl.AppendLine();
                int i = 0;
                foreach (var font in fonts)
                {
                    var nativeFont = font.CreateNativeFont();
                    tl.Append($"{i + 1}:\tベースフォント: {font.BaseFont} 埋め込み: {font.IsEmbedded}");
                    tl.AppendParagraphBreak();
                    if (nativeFont != null)
                        tl.AppendLine($"\tFontオブジェクトの作成: 成功 \n\tフォントファミリー: {nativeFont.FontFamilyName}、太字: {nativeFont.FontBold}、斜体: {nativeFont.FontItalic}");
                    else
                        tl.AppendLine("\tFontオブジェクトの作成: 失敗");
                    tl.AppendLine();
                    ++i;
                }
                tl.PerformLayout(true);
                while (true)
                {
                    // 'rest' は、収まりきらなかったテキストを受け入れます。
                    var splitResult = tl.Split(to, out TextLayout rest);
                    doc.Pages.Last.Graphics.DrawTextLayout(tl, PointF.Empty);
                    if (splitResult != SplitResult.Split)
                        break;
                    tl = rest;
                    doc.NewPage();
            }
            // PDF ドキュメントを保存します。
            doc.Save(stream);
            return doc.Pages.Count;
        }
    }
}
